/**
    Author: Picchiri Alessandro 
	Date: 2019.12.02
	File: NexiPayMailMng.js
*/

'use strict';
/* General Scripts */
var ISML = require('dw/template/ISML');
var guard = require('~/cartridge/scripts/guard');
var BackofficeSVC = require("~/cartridge/scripts/svc/XPayBackoffice");
var XPayUtils = require("~/../int_nexixpay/cartridge/scripts/util/XPayUtils");

/* Logger */
var logger = dw.system.Logger.getLogger('nexi.xpay.bm');

/**
 * Call to Nexi for to request paymail url and for to request paymail report 
 * The call for the report request is always executed
 * The request for the payment url is required only if it comes from the payment order
 * @link https://ecommerce.nexi.it/specifiche-tecniche/apibackoffice/richiestalinkpaymail.html
 * @link https://ecommerce.nexi.it/specifiche-tecniche/apibackoffice/ReportPaymail.html
 * @param args
 * @returns
 */
function callPayMail() {

    var PayMailSvc = require("~/cartridge/scripts/svc/PayMailSvc.js");

    var args = {};
    args.errorMsg = '';

    if(args.view !== 'reportPayMail'){ 

        if(request.httpParameterMap.orderNum) {
            args.orderLocale = '';
            var order = dw.order.OrderMgr.getOrder(request.httpParameterMap.orderNum.stringValue);            
            
            if(order) {
                var orderCustomerLocale =  order.customer.profile.preferredLocale;  
                args.orderLocale = orderCustomerLocale;

                args.orderNum = order.orderNo;        
                args.orderToken = order.orderToken;                    

                args.view = request.httpParameterMap.view.stringValue; // lastPayMailUrl / reportPayMail        
                args.importo = request.httpParameterMap.importo.stringValue;
                args.transactionCode = XPayUtils.generateUUIDCode();        
                
                var getLinkMail = PayMailSvc.getLinkPayMail(args)
                
                var result = JSON.parse(getLinkMail.object);
                result.transactionCode = args.transactionCode;        

                if(result.esito == 'KO'){
                    logger.debug('NexiPayMailMng-callPayMail PayMail request failed: ' + result.errore.messaggio); 
                    args.errorMsg = result.errore.messaggio;
                }
            }
        }        

    }
    // END request payment url
    
    // START / END report request
    return showPayMailLinks(result);
}


/**
 * controller's logic to gets all links of the report
 * @returns
 */
function showPayMailLinks(lastPayMailURL) {

    var topUrl = dw.web.URLUtils.url('SiteNavigationBar-ShowMenuitemOverview', 'CurrentMenuItemId', 'XPayMenuItem01');
    var mainMenuName = dw.web.Resource.msg('nexi.bo.mainmenu', 'backoffice', null);
        
    session.forms.boreportsearch.bc_root_name.value = mainMenuName;
    session.forms.boreportsearch.bc_root_link.value = topUrl;

    //init search dates
    var today = new Date();
    var beforeDate = new Date((new Date()).setDate(today.getDate()-7)); // 1 week before
    session.forms.boreportsearch.toDate.value = session.forms.boreportsearch.toDate.value || today.toISOString().substring(0,10);
    session.forms.boreportsearch.fromDate.value = session.forms.boreportsearch.fromDate.value || beforeDate.toISOString().substring(0,10);
   
    var args = {};
    args.errorMsg = '';
    args.infoMsg = '';
    args.fromDate = BackofficeSVC.getNexiDateFromString(session.forms.boreportsearch.fromDate.value);
    args.toDate = BackofficeSVC.getNexiDateFromString(session.forms.boreportsearch.toDate.value);

    args.topUrl = session.forms.boreportsearch.bc_root_link.value;
	args.mainMenuName = session.forms.boreportsearch.bc_root_name.value;
    
    //call to XPay to retrieve a list of paymail links
    var result = getXPayReport(args);
    
    if(result.esito === 'OK'){

        args.pmLinks = result.paymail;
        		
		if(empty(args.pmLinks)) {            
			args.infoMsg = dw.web.Resource.msg('reports.noresultsfound', 'backoffice', null);
            logger.warn(args.infoMsg);  
		}
    } 
    else {
        args.pmLinks = [];
		args.errorMsg =  result.errore.messaggio +' ('+result.errore.codice+')';
        logger.error('NexiPayMailMng-showPayMailLinks result: '+result.errore.codice+', '+result.errore.messaggio);
    }
    
    return renderPayMailReport(args,lastPayMailURL);
}



/**
 * Handles the call for links list (see BackofficeSVC.callReportsList() for details on params)
 * @param args <Object>
 * @returns JSON result
 */
function getXPayReport(args) {
	try {

        var params = {
            fromDate: args.fromDate,
            toDate: args.toDate
        };
                
        var httpResult = BackofficeSVC.callPayMailReportList(params);
        
		if(!httpResult.ok) {			
			throw httpResult;
		}
		
		var jsonResponse = JSON.parse(httpResult.object);
		logger.debug('callPayMailReportList-jsonResponse:\n'+JSON.stringify(jsonResponse, null, '\t'));

        return validateResultMac(jsonResponse);
	} 
	catch(ex) {
		logger.error('Eccezione: '+JSON.stringify(ex));
		return {
			esito: 'KO',
			errore: {
				codice: ex.error,
				messaggio: ex.errorMessage,
			}
		}
	}
}

/**
 * Validates the result of XPay services by checking the "mac" field
 * @param jsonResponse - XPay result object
 * @returns
 */
function validateResultMac(jsonResponse) {

	var macCalculated = XPayUtils.sha1(
		'esito=' + jsonResponse.esito + 
		'idOperazione=' + jsonResponse.idOperazione + 
		'timeStamp=' + jsonResponse.timeStamp + 
		XPayUtils.XPAY_TERM_WEB_SECRETKEY);
	
	if(jsonResponse.mac != macCalculated) {
		throw { 
			errorMessage: dw.web.Resource.msg('reports.macNotValidated', 'backoffice', null),
			error: '-',
		}
	}
	return jsonResponse; 
}


/**
 * Renders the NexiPayReportSearch template with all related args
 * @param args
 * @returns
 */
function renderPayMailReport(args,lastPayMailURL) {

    args = args || {};

    return ISML.renderTemplate('NexiPayMailReportSearch',{
        errorMsg 		: args.errorMsg || '',
        pmLinks 	    : args.pmLinks || [],
        lastPayMailURL  : lastPayMailURL,
        infoMsg         : args.infoMsg,
        TOP_URL         : args.topUrl,
        MAIN_MENU_NAME  : args.mainMenuName,
        view            : request.httpParameterMap.view.stringValue
    });
}


exports.CallPayMail = guard.ensure(['https'], callPayMail); 