'use strict';

/* General Scripts */
var ISML = require('dw/template/ISML');
var guard = require('~/cartridge/scripts/guard');

/* Logger */
var logger = dw.system.Logger.getLogger('nexi.xpay.bm');

/* XPay scripts from int_nexixpay and bm_nexixpay cartridges */
var XPayUtils = require("~/../int_nexixpay/cartridge/scripts/util/XPayUtils");
var BackofficeSVC = require("~/cartridge/scripts/svc/XPayBackoffice");


/**
 * Initialize the search reports page
 * @returns
 */
function start() {
	session.forms.boreportsearch.refDate.value = '';

	var topUrl = dw.web.URLUtils.url('SiteNavigationBar-ShowMenuitemOverview', 'CurrentMenuItemId', request.httpParameterMap.CurrentMenuItemId);
	if(!request.httpParameterMap.CurrentMenuItemId.value) {
		topUrl = request.httpParameterMap.TopUrl.value;
	}
	
    var mainMenuName = request.httpParameterMap.mainmenuname.stringValue;
    session.forms.boreportsearch.bc_root_name.value = mainMenuName;
    session.forms.boreportsearch.bc_root_link.value = topUrl;
    //session.forms.boreportsearch.refDate.value = (new Date()).toISOString().substring(0,10);	//default to today
    
	return renderSearch({
		topUrl: topUrl,
		mainMenuName: mainMenuName
	});	
}

/**
 * Handles form actions (search and reimbursement actions)
 * @returns
 */
function handleForm() {
	
	session.forms.boreportsearch.refDate.value = session.forms.boreportsearch.refDate.value || '';
	
	var triggeredAction = request.triggeredFormAction;
	if (!empty(triggeredAction)){
		if(triggeredAction.formId === 'search') {
			return getReports();
		}
	}
	return start();
}

/**
 * Renders the NexiPayReportSearch template with all related args
 * @param args
 * @returns
 */
function renderSearch(args) {
	args = args || {};

    return ISML.renderTemplate('NexiPayReportSearch', {
        ContinueURL     : dw.web.URLUtils.https('NexiPayReportMng-HandleForm'),
        CurrentForms    : session.forms,
        reports 	    : args.reports || [],
        TOP_URL         : args.topUrl,
        MAIN_MENU_NAME  : args.mainMenuName,
        errorMsg 		: args.errorMsg || '',
        infoMsg 		: args.infoMsg || ''
    });
}

/**
 * Handles the call for reports list (see BackofficeSVC.callReportsList() for details on params)
 * @param args <Object>
 * @returns JSON result
 */
function getXPayReports(args) {
	try {
		var params = {
			refDate: args.refDate
		};
		var httpResult = BackofficeSVC.callReportList(params);
		if(!httpResult.ok) {			
			throw httpResult;
		}
		
		var jsonResponse = JSON.parse(httpResult.object);
		logger.debug('callReportList-jsonResponse:\n'+JSON.stringify(jsonResponse, null, '\t'));
		
		return validateResultMac(jsonResponse);
	} 
	catch(ex) {
		logger.error('Eccezione: '+JSON.stringify(ex));
		return {
			esito: 'KO',
			errore: {
				codice: ex.error,
				messaggio: ex.errorMessage,
			}
		}
	}
}


/**
 * Validates the result of XPay services by checking the "mac" field
 * @param jsonResponse - XPay result object
 * @returns
 */
function validateResultMac(jsonResponse) {
	var macCalculated = XPayUtils.sha1(
		'esito=' + jsonResponse.esito + 
		'idOperazione=' + jsonResponse.idOperazione + 
		'timeStamp=' + jsonResponse.timeStamp + 
		XPayUtils.XPAY_TERM_WEB_SECRETKEY);
	
	if(jsonResponse.mac != macCalculated) {
		throw { 
			errorMessage: dw.web.Resource.msg('reports.macNotValidated', 'backoffice', null),
			error: '-',
		}
	}
	return jsonResponse;
}


/**
 * Transform a yyyy-mm-dd date string into dd/mm/yyyy 
 * @param val <String> yyyy-mm-dd
 * @returns <String> dd-mm-yyyy
 */
function getNexiDateFromString(val) {
	if(!val) return null;
	if(val.length !== 10) return null;
	var tmp = val.split('-');
	return tmp[2]+'/'+tmp[1]+'/'+tmp[0];
}


/**
 * Gets all the reports controller's logic
 * @returns
 */
function getReports() {
	
	var args = {};
	args.errorMsg = '';
	args.infoMsg = '';
	args.refDate = getNexiDateFromString(session.forms.boreportsearch.refDate.value);
	args.topUrl = session.forms.boreportsearch.bc_root_link.value;
	args.mainMenuName = session.forms.boreportsearch.bc_root_name.value;
	
	var baseUrl = BackofficeSVC.getServiceBaseUrl() + "/ecomm/bo/dManager/downloadReport";

	//call to Nexi for list of reports
	var result = getXPayReports(args);
	if(result.esito === 'OK'){
		args.reports = result.listaReport;
		
		//for any report, create a url link to download the report
		for(var i = 0; i < args.reports.length; i++){
			var rep = args.reports[i];
			
			var timeStamp = (new Date()).getTime();
			var mac = createDownloadMac(timeStamp, rep.id);
			rep.actionLink = baseUrl + "?apiKey="+XPayUtils.XPAY_API_KEY+"&timeStamp="+timeStamp+"&idReport="+rep.id+"&mac="+mac;
		}

		if(empty(args.reports)) {
			args.infoMsg = dw.web.Resource.msg('reports.noresultsfound', 'backoffice', null);
		}
	} else {
		args.reports = [];
		args.errorMsg =  result.errore.messaggio +' ('+result.errore.codice+')';
		logger.error('ERROR: '+result.esito.codice+', '+result.esito.messaggio);
	}
	
	return renderSearch(args);
}

/**
 * create a mac to validate a report download request
 * {@link https://ecommerce.nexi.it/specifiche-tecniche/apibackoffice/report/download.html}
 * 
 * @param timeStamp - a timestamp in millisecons
 * @param idReport - a report id
 * @returns {String} a mac string
 */
function createDownloadMac(timeStamp, idReport) {
	var mac = XPayUtils.sha1(
		'apiKey=' +XPayUtils.XPAY_API_KEY + 
        'timeStamp=' + timeStamp + 
        'idReport=' + idReport + 
        XPayUtils.XPAY_TERM_WEB_SECRETKEY);	
	return mac;
}


exports.Start 				= guard.ensure(['https'], start);
exports.HandleForm 			= guard.ensure(['https', 'post'], handleForm);