'use strict';

/* General Scripts */ 
var ISML = require('dw/template/ISML');
var guard = require('*/cartridge/scripts/guard');
var app = require('*/cartridge/scripts/app');
var Resource = require('dw/web/Resource');
var Order = require('dw/order/Order');
var OrderMgr = require('dw/order/OrderMgr');

var XPayUtils = require('~/cartridge/scripts/util/XPayUtils');

/* Logger */
var logger = dw.system.Logger.getLogger('nexi.xpay.int');


/**
 * Call back method coming from url parameter
 * 
 * Once the payment has been completed, the customer is redirected to the merchant site at 
 * the address indicated in the payment initiation message ("url" field). The user then returns 
 * to  the  merchant’s  site,  bringing  the  parameters  that  attest  to  the  conclusion  of  the 
 * transaction.
 * {@link https://ecommerce.nexi.it/specifiche-tecniche/codicebase/esito.html}

 * @param {Object} context context object used for the view
 */
function payOutcome(context) {

    logger.info("\nXPayCallback.PayOutcome");

    var params = request.httpParameterMap;
    session.custom.alias = params.alias.value;
    session.custom.importo = params.importo.value;
    session.custom.divisa = params.divisa.value;
    session.custom.codTrans = params.codTrans.value;
    session.custom.brand = params.brand.value;
    session.custom.mac = params.mac.value;
    session.custom.esito = params.esito.value;
    session.custom.data = params.data.value;
    session.custom.orario = params.orario.value;
    session.custom.codiceEsito = params.codiceEsito.value;
    session.custom.codAut = params.codAut.value;
    session.custom.pan = params.pan.value;
    session.custom.scadenza_pan = params.scadenza_pan.value;
    session.custom.regione = params.regione.value;
    session.custom.nazionalita = params.nazionalita.value;
    session.custom.descrizione = params.descrizione.value;
    session.custom.languageId = params.languageId.value;
    session.custom.tipoTransazione = params.tipoTransazione.value;
    session.custom.tipoProdotto = params.tipoProdotto.value;
    session.custom.nome = params.nome.value;
    session.custom.cognome = params.cognome.value;
    session.custom.messaggio = params.messaggio.value;
    session.custom.mail = params.mail.value;
    session.custom.hash = params.hash.value;
    session.custom.infoc = params.infoc.value;
    session.custom.infob = params.infob.value;
    session.custom.codiceConvenzione = params.codiceConvenzione.value;

    //oneclick params
    session.custom.num_contratto = params.num_contratto.value;
    session.custom.tipo_servizio = params.tipo_servizio.value;
    session.custom.gruppo = params.gruppo.value;
    session.custom.Check = params.Check.value;

    //additional parameters
    session.custom.orderNum = params.orderNum.value;
    session.custom.orderToken = params.orderToken.value;

    //retrieve order created before redirect to nexi payment page
    var orderNum = request.httpParameterMap.orderNum.value;
    var orderToken = request.httpParameterMap.orderToken.value;
    var order = empty(orderToken) ? OrderMgr.getOrder(orderNum) :
                                    OrderMgr.getOrder(orderNum, orderToken);
    
    if(!empty(order)) {        
        logger.debug("XPayCallback.PayOutcome-orderNo: {0} status: {1}", order.orderNo, order.status);
                    
        //if S2S is disabled we need to handle the order here
        if(!XPayUtils.XPAY_S2S_ENABLED) {
            
            var placeOrderResult = require('~/cartridge/controllers/XPayOrderMng').handleOrder(order);        
            // a valle di questo l'ordine sarà OPEN oppure FAILED
            
            //questo codice viene direttamente da COSummary-Submit (che però è una form submit in POST con csrf)
            if(placeOrderResult.error) {                
                //order status = FAILED
                renderOrderOutcome(order, Resource.msg('xpay.placeorder.order_failed','xpay',null));
                logger.debug("XPayCallback.PayOutcome order failed.");
            }
            else if(placeOrderResult.order_created) {
                //order status = OPEN
                var COSummary = require('*/cartridge/controllers/COSummary');
                COSummary.ShowConfirmation(order);            
                logger.debug("XPayCallback.PayOutcome-Order succefully opened - orderNo: {0} status: {1}", order.orderNo, order.status);
            }
        }
        //...otherwise the order would be handled in Notify callback
        else {            
            //just show proper page here, no order handling required
            if(session.custom.esito !== 'OK' || order.status == Order.ORDER_STATUS_FAILED) {
                //FAIL ORDER status would be set in notify 
                renderOrderOutcome(order, Resource.msg('xpay.placeorder.order_failed','xpay',null));
                logger.debug("XPayCallback.PayOutcome order failed.");
            }
            else {
                var COSummary = require('*/cartridge/controllers/COSummary');
                COSummary.ShowConfirmation(order);            
                logger.debug("XPayCallback.PayOutcome order submitted.");
            }
        }            
    }    
}

/**
 * Call back method coming from url_back parameter
 * 
 * If a customer decides to cancel the payment from the Nexi check-out page by using the 
 * appropriate  cancellation  button,  or  if  an  error  occurs  during  the  payment  process,  the 
 * customer  will  be  redirected  to  the  url  indicated  in the  "url_back"  parameter  during  the 
 * payment initiation process, along with the additional parameters.
 * {@link https://ecommerce.nexi.it/specifiche-tecniche/codicebase/annullo.html}
 */
function payCancel() {
    logger.info("\nXPayCallback.PayCancel");
    
    var params = request.httpParameterMap;
    session.custom.alias = params.alias.value;
    session.custom.importo = params.importo.value;
    session.custom.divisa = params.divisa.value;
    session.custom.codTrans = params.codTrans.value;
    session.custom.esito = params.esito.value;
    
    logger.debug("Easy Payment cancelled - outcome "+session.custom.esito);
        
    var orderNum = request.httpParameterMap.orderNum.value;
    var orderToken = request.httpParameterMap.orderToken.value;    

    var order = empty(orderToken) ? OrderMgr.getOrder(orderNum) :
                                    OrderMgr.getOrder(orderNum, orderToken);

    dw.system.Transaction.wrap(function () {
        //OrderMgr.placeOrder(order);
        //OrderMgr.cancelOrder(order); 
        OrderMgr.failOrder(order, true);
        OrderMgr.undoFailOrder(order);
        OrderMgr.placeOrder(order);
        OrderMgr.cancelOrder(order);   
        //OrderMgr.undoFailOrder(order);
    });
    
    require('*/cartridge/controllers/COSummary').Start();        
    //renderOrderOutcome(order, Resource.msg('xpay.placeorder.order_canceled','xpay',null));
}
 
/**
 * Call back method coming from urlpost parameter
 * 
 * The merchant receives payment notification directly from the Nexi server through a POST call.  
 * The notification is sent to the address indicated in the "urlpost" parameter of the 
 * Payment Initiation Message. 
 * WARNING: 
 * To confirm receipt of the notification, the message returned from the call must be a "http 200"
 * {@link https://ecommerce.nexi.it/specifiche-tecniche/codicebase/notifica.html}
 */
function notify() {
    logger.info("\nXPayCallback.Notify");

    if(!XPayUtils.XPAY_S2S_ENABLED) {
        logger.info("\nXPayCallback.Notify-S2S disabled, skipping notify.");
        return;
    }
    
    //retrieve all parameters from Nexi post http callback
    var params = request.httpParameterMap;
    var alias = params.alias.value;
    var importo = params.importo.value;
    var divisa = params.divisa.value;
    var codTrans = params.codTrans.value;
    var brand = params.brand.value;
    var mac = params.mac.value;
    var esito = params.esito.value;
    var data = params.data.value;
    var orario = params.orario.value;
    var codiceEsito = params.codiceEsito.value;
    var codAut = params.codAut.value;
    var pan = params.pan.value;
    var scadenza_pan = params.scadenza_pan.value;
    var regione = params.regione.value;
    var nazionalita = params.nazionalita.value;
    var descrizione = params.descrizione.value;
    var languageId = params.languageId.value;
    var tipoTransazione = params.tipoTransazione.value;
    var tipoProdotto = params.tipoProdotto.value;
    var nome = params.nome.value;
    var cognome = params.cognome.value;
    var messaggio = params.messaggio.value;
    var mail = params.mail.value;
    var hash = params.hash.value;
    var infoc = params.infoc.value;
    var infob = params.infob.value;
    var codiceConvenzione = params.codiceConvenzione.value;

    //put all Nexi payment data into the session
    session.custom.alias = params.alias.value;
    session.custom.importo = params.importo.value;
    session.custom.divisa = params.divisa.value;
    session.custom.codTrans = params.codTrans.value;
    session.custom.brand = params.brand.value;
    session.custom.mac = params.mac.value;
    session.custom.esito = params.esito.value;
    session.custom.data = params.data.value;
    session.custom.orario = params.orario.value;
    session.custom.codiceEsito = params.codiceEsito.value;
    session.custom.codAut = params.codAut.value;
    session.custom.pan = params.pan.value;
    session.custom.scadenza_pan = params.scadenza_pan.value;
    session.custom.regione = params.regione.value;
    session.custom.nazionalita = params.nazionalita.value;
    session.custom.descrizione = params.descrizione.value;
    session.custom.languageId = params.languageId.value;
    session.custom.tipoTransazione = params.tipoTransazione.value;
    session.custom.tipoProdotto = params.tipoProdotto.value;
    session.custom.nome = params.nome.value;
    session.custom.cognome = params.cognome.value;
    session.custom.messaggio = params.messaggio.value;
    session.custom.mail = params.mail.value;
    session.custom.hash = params.hash.value;
    session.custom.infoc = params.infoc.value;
    session.custom.infob = params.infob.value;
    session.custom.codiceConvenzione = params.codiceConvenzione.value;

    //oneclick params
    session.custom.num_contratto = params.num_contratto.value;
    session.custom.tipo_servizio = params.tipo_servizio.value;
    session.custom.gruppo = params.gruppo.value;
    session.custom.Check = params.Check.value;

    //additional parameters
    session.custom.orderNum = params.orderNum.value;

    logger.info("\nXPayCallback-Notify-esito: {0} - codiceEsito: {1}", esito, codiceEsito);    
    //se OK si effettua una verifica che la richiesta abbia un senso lato Nexi

    var orderNum = params.orderNum.value;    
    var orderToken = params.orderToken.value;    

    var order = empty(orderToken) ? OrderMgr.getOrder(orderNum) :
                                    OrderMgr.getOrder(orderNum, orderToken);

    //check esito and MAC...
    var placeOrderResult = require('~/cartridge/controllers/XPayOrderMng').handleOrder(order);
    // a valle di questo l'ordine sarà OPEN oppure FAILED
    
    //questo codice viene direttamente da COSummary-Submit (che però è una form submit in POST con csrf)
    if(placeOrderResult.error) {  
        //enter here if esito = KO or mac non valid or unepected exception in payment processor
        //order status = FAILED      
        logger.debug("XPayCallback-Notify-placeOrderResult.error: "+placeOrderResult.error);            
    }
    else if(placeOrderResult.order_created) {        
        //enter here if everything is ok
        //order status = OPEN      
        var order = placeOrderResult.Order;
        logger.debug("XPayCallback-Notify-Order created - orderNo: {0} status: {1}", order.orderNo, order.status);
    }    
    
    //Per confermare l'avvenuta ricezione della notifica il messaggio restituito dalla chiamata deve essere un "HTTP 200".
    response.setStatus(200);
}
 

function renderOrderOutcome(order, msg) {
    var pageMeta = require('*/cartridge/scripts/meta');
    pageMeta.update({pageTitle: dw.web.Resource.msg('confirmation.meta.pagetitle', 'checkout', 'SiteGenesis Checkout Confirmation')});
    app.getView({
        Order: order,
        OrderMsg: msg,
        ContinueURL: dw.web.URLUtils.https('Account-RegistrationForm') // needed by registration form after anonymous checkouts
    }).render('checkout/confirmation/order_outcome');
}



/*
* Web exposed methods
*/
exports.PayOutcome = guard.ensure(['https'], payOutcome); 
exports.PayCancel = guard.ensure(['https'], payCancel); 
exports.Notify = guard.ensure(['https'], notify); 
