'use strict';

var XPayUtils = require('~/cartridge/scripts/util/XPayUtils');
var logger = dw.system.Logger.getLogger('nexi.xpay.int');

/**
  * call a Nexi API RESTful api to perform a basic xpay build payment
  * 
  * for params json specs see:
  * {@link https://ecommerce.nexi.it/specifiche-tecniche/build/pagamento.html}
  * 
  * @returns {Result} a json response
  */
 function payNonce(params) {
    
    params = params || {};
    params.timeStamp = (new Date()).getTime();
    
    params.mac = XPayUtils.sha1(
        'apiKey=' +XPayUtils.XPAY_API_KEY + 
        'codiceTransazione=' + params.codiceTransazione + 
        'importo=' + params.importo + 
        'divisa=' + params.divisa + 
        'xpayNonce=' + params.xpayNonce + 
        'timeStamp=' + params.timeStamp + 
        XPayUtils.XPAY_TERM_WEB_SECRETKEY);
    
    //obtains a Service object by calling the LocalServiceRegistry.createService method. 
    //The method takes two arguments: the name of a service configuration, 
    //and a script object that contains callback code that you provide	 
    var pagaNonceWs = dw.svc.LocalServiceRegistry.createService("Nexi.XPay.restservice", {
        createRequest: function(svc: HTTPService, args) {
            svc.setRequestMethod("POST");
            if(args) {
                logger.debug('\nPayNonceSvc - request {0} :\n{1}', this.URL, JSON.stringify(args, null, '\t'));
                svc.addHeader("Content-Type", "application/json");            	
                return JSON.stringify(args);
            }            
        },
        parseResponse: function(svc: HTTPService, client: HTTPClient) {
            return client.text;
        },
        mockCall: function(svc: HTTPService, client: HTTPClient) {
            return {
                statusCode: 200,
                statusMessage: "Success",
                text: '{"esito":"OK","idOperazione":"1108720","timeStamp":"1568110121373","mac":"f4bf146a2953934eb3b3412680a5694c0256c7cc","codiceAutorizzazione":"L1U5QQ","data":"10/09/2019","ora":"12:08:40","regione":"","nazione":"ITA","tipoProdotto":"VISA CLASSIC - CREDIT - N","ppo":"","codiceConvenzione":"00018210104","brand":"VISA","tipoTransazione":"VBV_FULL"}'
            };
        }
    });
    pagaNonceWs.URL += "/ecomm/api/hostedPayments/pagaNonce";
    
    var result : Result = pagaNonceWs.call(params);
    
    return result;
}

/**
  * call the XPay API RESTful api to perform a first recurring payment with xpay build payment method
  * 
  * for params json specs see:
  * {@link https://ecommerce.nexi.it/specifiche-tecniche/pagamentoricorrente/primopagamento.html}
  * 
  * @returns {Result} a json response
  */
 function firstRecurPay(params) {
    
    params = params || {};
    params.timeStamp = (new Date()).getTime();
        
    //apiKey=<val>codiceTransazione=<val>importo=<val>divisa=<val>xpayNonce=<val>timeStamp=<val><chiaveSegreta>
    params.mac = XPayUtils.sha1(
        'apiKey=' +params.apiKey + 
        'codiceTransazione=' + params.codiceTransazione + 
        'importo=' + params.importo + 
        'divisa=' + params.divisa + 
        'xpayNonce=' + params.xpayNonce + 
        'timeStamp=' + params.timeStamp + 
        XPayUtils.XPAY_TERM_WEB_SECRETKEY);
    
    //obtains a Service object by calling the LocalServiceRegistry.createService method. 
    //The method takes two arguments: the name of a service configuration, 
    //and a script object that contains callback code that you provide	 
    var recurPayWs = dw.svc.LocalServiceRegistry.createService("Nexi.XPay.restservice", {
        createRequest: function(svc: HTTPService, args) {
            svc.setRequestMethod("POST");
            if(args) {
                logger.debug('\nPayNonceSvc - request {0} :\n{1}', this.URL, JSON.stringify(args, null, '\t'));
                svc.addHeader("Content-Type", "application/json");            	
                return JSON.stringify(args);
            }            
        },
        parseResponse: function(svc: HTTPService, client: HTTPClient) {
            return client.text;
        }
    });
    recurPayWs.URL += "/ecomm/api/hostedPayments/pagaNonceCreazioneContratto";
    
    var result : Result = recurPayWs.call(params);
    
    return result;
}


exports.payNonce = payNonce;
exports.firstRecurPay = firstRecurPay;
