'use strict';

/* General Scripts */
var Transaction = require('dw/system/Transaction');
var PaymentMgr = require('dw/order/PaymentMgr');

/* Logger */
var logger = dw.system.Logger.getLogger('nexi.xpay.int');

/* XPay scripts from int_nexixpay and bm_nexixpay cartridges */
var XPayUtils = require("~/cartridge/scripts/util/XPayUtils");
var RecurPaySvc = require("~/../bm_nexixpay/cartridge/scripts/svc/RecurPaySvc");

/**
 * execute a subsequent payment on an recurring order 
 * @param {dw.order.Order} order an order instance of a recurring order
 * @returns a js object containig the jsonRequest and jsonResponse of the rest webservice
 */
function execRecurPay(order) {
    
    var jsonReq = {
        apiKey              : XPayUtils.XPAY_TERM_RICO_ALIAS,
        numeroContratto     : order.custom.xpay_recurContractNum,
        codiceTransazione   : XPayUtils.generateUUIDCode(),
        importo             : (order.totalGrossPrice.value*100).toFixed(),
        divisa              : XPayUtils.XPAY_CURRENCY,
        codiceGruppo        : XPayUtils.XPAY_TERM_RICO_GROUP,
        scadenza            : '',
        timeStamp           : (new Date()).getTime(),
        parametriAggiuntivi : {             
            TCONTAB  : XPayUtils.XPAY_TCONTAB
        }
    }

    var httpResult = RecurPaySvc.subseqPay(jsonReq);
    
    if(!httpResult.ok) throw httpResult;

    var jsonRes = JSON.parse(httpResult.object);
    logger.debug('XPayRecurTester-jsonRes:\n'+JSON.stringify(jsonRes, null, '\t'));	
            
    saveTransaction(order, jsonReq, jsonRes);

    return {
        jsonRequest : jsonReq,
        jsonResponse: jsonRes
    }        
}


/**
 * update payment transaction data model
 * 
 * @param {dw.order.Order} Order a recurring order
 * @param {String} jsonReq json request 
 * @param {String} jsonResponse json response
 */
function saveTransaction(order, jsonReq, jsonResponse) {
	    
    Transaction.wrap(function() {
        
        var paymentInstrument = order.createPaymentInstrument(XPayUtils.XPAY_PAYINST, order.totalGrossPrice);
        var paymentProcessor = PaymentMgr.getPaymentMethod(paymentInstrument.getPaymentMethod()).getPaymentProcessor();
        paymentInstrument.paymentTransaction.paymentProcessor = paymentProcessor;

        var result = jsonResponse.esito;
        var idOperazione = jsonResponse.idOperazione;
        var mac = jsonResponse.mac;		
        var timeStamp = jsonResponse.timeStamp;		
        var transactionCode = jsonReq.codiceTransazione;
        
        order.custom.xpay_transactionCode = transactionCode;
        
        paymentInstrument.paymentTransaction.transactionID = transactionCode;        
        paymentInstrument.paymentTransaction.custom.xpay_esito = result;
        //paymentInstrument.paymentTransaction.custom.xpay_codiceEsito = resultCode;
        paymentInstrument.paymentTransaction.custom.xpay_idOperazione = idOperazione;
        paymentInstrument.paymentTransaction.custom.xpay_mac = mac;
        paymentInstrument.paymentTransaction.custom.xpay_timeStamp = timeStamp;

        if(result !== 'OK') {		
            paymentInstrument.paymentTransaction.custom.xpay_codiceErrore = jsonResponse.errore.codice;
            paymentInstrument.paymentTransaction.custom.xpay_messaggioErrore = jsonResponse.errore.messaggio;
        }
        else {
            //transaction data
            paymentInstrument.paymentTransaction.custom.xpay_alias = jsonReq.apiKey;            
            paymentInstrument.paymentTransaction.custom.xpay_codiceAutorizzazione = jsonResponse.codiceAutorizzazione;
            paymentInstrument.paymentTransaction.custom.xpay_codiceConvenzione = jsonResponse.codiceConvenzione;
            paymentInstrument.paymentTransaction.custom.xpay_data = jsonResponse.data;
            paymentInstrument.paymentTransaction.custom.xpay_ora = jsonResponse.ora;
            paymentInstrument.paymentTransaction.custom.xpay_nazione = jsonResponse.nazione;
            //paymentInstrument.paymentTransaction.custom.xpay_descrizione = description;
            //paymentInstrument.paymentTransaction.custom.xpay_languageId = languageId;
            paymentInstrument.paymentTransaction.custom.xpay_regione = jsonResponse.regione;
            paymentInstrument.paymentTransaction.custom.xpay_tipoTransazione = jsonResponse.tipoTransazione;
            paymentInstrument.paymentTransaction.custom.xpay_tipoProdotto = jsonResponse.tipoProdotto;
            //paymentInstrument.paymentTransaction.custom.xpay_messaggio = transacMsg;
            //paymentInstrument.paymentTransaction.custom.xpay_mail = mail;
            //paymentInstrument.paymentTransaction.custom.xpay_infoc = infoc;
            //paymentInstrument.paymentTransaction.custom.xpay_infob = infob;
            paymentInstrument.paymentTransaction.custom.xpay_tcontab = XPayUtils.XPAY_TCONTAB;
    
            //card data
            if(jsonResponse.dettagliCarta) {                
                var panExpiry =  jsonResponse.dettagliCarta.expiry;                

                //paymentInstrument.creditCardHolder = name + ' ' + surname;
                paymentInstrument.creditCardNumber = jsonResponse.dettagliCarta.maskedPan;
                paymentInstrument.creditCardType = jsonResponse.brand;
                paymentInstrument.creditCardExpirationYear = panExpiry ? panExpiry.substring(0,4) : '';
                paymentInstrument.creditCardExpirationMonth = panExpiry ? panExpiry.substring(4,6) : '';

                paymentInstrument.paymentTransaction.custom.xpay_hash = jsonResponse.dettagliCarta.hash;
            }
            
            var terminal = XPayUtils.getTermByAlias(jsonReq.apiKey);
    
            // calculate mac with the merchant secret key
            var macCalculated = XPayUtils.sha1(
                'esito=' + result + 
                'idOperazione=' + idOperazione + 
                'timeStamp=' + timeStamp + 
                terminal.secretkey);			
    
            // ...and check if it matches with mac sent by Nexi
            if(macCalculated != mac) {
    
                var macErrMsg = "Invalid response: MAC not matching";
                if(!empty(paymentInstrument.paymentTransaction.custom.xpay_messaggioErrore))
                    paymentInstrument.paymentTransaction.custom.xpay_messaggioErrore += '; \n';
                else
                    paymentInstrument.paymentTransaction.custom.xpay_messaggioErrore = '';
                paymentInstrument.paymentTransaction.custom.xpay_messaggioErrore += macErrMsg;
                logger.error('\nXPayRecurTester: '+ macErrMsg);                
            }	
        }			
	});
}


exports.execRecurPay   = execRecurPay;